@@ -1,47 +1,105 @@
-use v5.26;
+package Blockchain::Ethereum::Transaction;
 
+use v5.26;
 use strict;
 use warnings;
-no indirect;
-use feature 'signatures';
 
-use Object::Pad ':experimental(init_expr)';
 # ABSTRACT: Ethereum transaction abstraction
-
-package Blockchain::Ethereum::Transaction;
-role Blockchain::Ethereum::Transaction;
-
 our $AUTHORITY = 'cpan:REFECO';    # AUTHORITY
-our $VERSION   = '0.010';          # VERSION
+our $VERSION   = '0.011';          # VERSION
 
 use Carp;
 use Crypt::Digest::Keccak256 qw(keccak256);
 
 use Blockchain::Ethereum::RLP;
 
-field $chain_id :reader :writer :param;
-field $nonce :reader :writer :param;
-field $gas_limit :reader :writer :param;
-field $to :reader :writer :param    //= '';
-field $value :reader :writer :param //= '0x0';
-field $data :reader :writer :param  //= '';
-field $v :reader :writer :param = undef;
-field $r :reader :writer :param = undef;
-field $s :reader :writer :param = undef;
+sub new {
+    my ($class, %args) = @_;
 
-field $rlp :reader = Blockchain::Ethereum::RLP->new();
+    my $self = bless {}, $class;
 
-method serialize;
+    foreach (qw(chain_id nonce gas_limit to value data v r s)) {
+        $self->{$_} = $args{$_} if exists $args{$_};
+    }
 
-method generate_v;
+    return $self;
+}
 
-method hash {
+sub rlp {
+    my $self = shift;
+
+    return $self->{rlp} //= Blockchain::Ethereum::RLP->new;
+}
+
+sub chain_id {
+    return shift->{chain_id};
+}
+
+sub nonce {
+    return shift->{nonce};
+}
+
+sub gas_limit {
+    return shift->{gas_limit};
+}
+
+sub to {
+    return shift->{to} // '';
+}
+
+sub value {
+    return shift->{value} // '0x0';
+}
+
+sub data {
+    return shift->{data} // '';
+}
+
+sub v {
+    return shift->{v};
+}
+
+sub set_v {
+    my ($self, $v) = @_;
+    $self->{v} = $v;
+}
+
+sub r {
+    return shift->{r};
+}
+
+sub set_r {
+    my ($self, $r) = @_;
+    $self->{r} = $r;
+}
+
+sub s {
+    my $self = shift;
+    return $self->{s};
+}
+
+sub set_s {
+    my ($self, $s) = @_;
+    $self->{s} = $s;
+}
+
+sub serialize {
+    croak "serialize method not implemented";
+}
+
+sub generate_v {
+    croak "generate_v method not implemented";
+}
+
+sub hash {
+    my $self = shift;
 
     return keccak256($self->serialize);
 }
 
 # In case of Math::BigInt given for any params, get the hex value
-method _equalize_params ($params) {
+sub _equalize_params {
+    my ($self, $params) = @_;
 
     return [map { ref $_ eq 'Math::BigInt' ? $_->as_hex : $_ } $params->@*];
 }
@@ -60,7 +118,7 @@ Blockchain::Ethereum::Transaction - Ethereum transaction abstraction
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =head1 SYNOPSIS
 
